/***********************************************************************
 * Bdbj 1.2 (by Jon Cook May 2002)
 *
 * BdbjBreakpoints.java 
 * 
 * Keep a track of the breakpoints and report them Kaffe.
 ***********************************************************************/

package jjc.bdbj.client;

public class Breakpoints {

    public static final int BREAKPOINTS = 15;
    private int[] breakpoints = new int[ BREAKPOINTS + 1 ];
    private String[] breakpointFile = new String[ BREAKPOINTS + 1 ];
    private ClientSocket clientSocket = null;

    public Breakpoints( ClientSocket clientSocket ) {
        this.clientSocket = clientSocket;
	initBreakpoints( );
    }

    public synchronized int getBreakpointLine( int index ) {
	return breakpoints[ index ];
    }

    public synchronized String getBreakpointFile( int index ) {
	return breakpointFile[ index ];
    }

    private synchronized void bpt( int index, int value, String file ) {
        breakpoints[ index ] = value;
	breakpointFile[ index ] = file;
    }

    private boolean isBreakpoint( int line, String file ) {
        for( int i = 0; i < BREAKPOINTS; i++ )
            if( breakpoints[ i ] == line 
		&& breakpointFile[ i ].equals( file ) ) 
                return true;
            else if( breakpoints[ i ] == -1 )
                return false;
        return false;
    }

    private void insertBreakpoint( int line, String file ) {
        for( int i = 0; i < BREAKPOINTS; i++ )
            if( breakpoints[ i ] == -1 ) {
                bpt( i, line, file ); 
                return;
            }
        clientSocket.writeLine( "error: Too Many Breakpoints" );
    }

    private void deleteBreakpoint( int line, String file ) {
        for( int i = 0; i < BREAKPOINTS; i++ )
            if( breakpoints[ i ] == line &&
		breakpointFile[ i ].equals( file ) ) {
                for( int j = i; j < BREAKPOINTS; j++ ) {
                    bpt( j, breakpoints[ j + 1 ], breakpointFile[ j + 1 ] );
                }
            }
    }

    public synchronized void clearAllBreakpoints( ) {
        while( breakpoints[ 0 ] != -1 )
            deleteBreakpoint( breakpoints[ 0 ], breakpointFile[ 0 ] );
    }

    private void initBreakpoints ( ) {
        for( int i = 0; i < BREAKPOINTS + 1; i++ ) 
            bpt( i, -1, "" );
    }

    public synchronized void toggleBreakpoint( int line, String file ) {
        if( isBreakpoint( line, file ) )
            deleteBreakpoint( line, file );
        else 
            insertBreakpoint( line, file );
    }

    public synchronized void listBreakpoints( ) {
        for( int i = 0; i < BREAKPOINTS; i++ ) {
            if( breakpoints[ i ] == -1 )
                break;
            else
                clientSocket.writeLine( "> " + breakpoints[ i ] + " " + breakpointFile[ i ] );
        }
	System.out.println( );
    }
}
