/***********************************************************************
 * Bdbj 1.2 (by Jon Cook May 2002)
 *
 * Client.java 
 * 
 * This program is executed by either the teletype or graphical UI
 * from which it receives commands via a socket which it passes on
 * to a modified version of Kaffe 1.0.6.
 ***********************************************************************/

package jjc.bdbj.client;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Vector;

public class Client {
    public static void main( String[] args ) {
        new Debugger( args );
    }
}

class Debugger {
    private ClientSocket clientSocket = null;
    private Interpreter interpreter = null;

    static void usage( ) {
        System.out.println( "It is not intended that the user run this program directly." );
        System.out.println( "It is invoked as Client <port> <host>, and connects to" );
        System.out.println( "the server socket on the given port and host." ); 
	System.out.println( );
	System.out.println( "The user interface entry points are:" );
	System.out.println( "  jjc.bdbj.interfaces.gui.Gui" );
	System.out.println( "  jjc.bdbj.interfaces.gui.WinGui" );
	System.out.println( "  jjc.bdbj.interfaces.gui.MotifGui" );
	System.out.println( "  jjc.bdbj.interfaces.tty.TTY" );
        System.exit( 1 );
    }

    private String[] helpText = {
    "Bdbj: Setup mode:",
    "-----------------",
    "class <string>   - set classname",
    "file <string>    - add classname to list of files to be debugged",
    "args <string>    - set arguments",
    "arg <string>     - set next argument (do not use in conjunction with args)",
    "init <integer> <string> - set initial method's first line and file classname",
    "buffer <integer> - set buffer size (roughly 8 buffer slots to 1 line of code)",
    "deeplog <string> - set the deep logging list",
    "",
    "toggle <integer> <string>  - toggle breakpoint status of line in file",
    "listbpts         - echo the table of breakpoints",
    "clearallbpts     - clear all breakpoints",
    "",
    "start            - finish setup and enter debugging mode",
    "end/exit/quit/terminate - abort debugging session",
    "help             - display this message"
    };

    private void help( ) {
        for( int i = 0; i < helpText.length; i++ )
            clientSocket.writeLine( helpText[ i ] );
    }
                            
    private String[] getStringArray( String str ) {
	str = str.trim( );
	String savedStr = str;
	int count = 0;
	for( ; ; ) {
	    int pos = str.indexOf( " " );
	    if( pos == -1 ) {
		if( str.length( ) != 0 ) 
		    count++;
		break;
	    }
	    count++;
	    str = str.substring( pos ).trim( );
	}
	str = savedStr;
	String[] strArray = new String[ count ];
	count = 0;
	for( ; ; ) {
	    int pos = str.indexOf( " " );
	    if( pos == -1 ) {
		if( str.length( ) != 0 )
		    strArray[ count ] = str;
		break;
	    }
	    strArray[ count ] = str.substring( 0, pos ).trim( );
	    count++;
	    str = str.substring( pos ).trim( );
	}
	return strArray;
    }

    private String[] vectorToStringArray( Vector vector ) {
	String[] result = new String[ vector.size( ) ];
	for( int i = 0; i < vector.size( ); i++ )
	    result[ i ] = vector.elementAt( i ).toString( );	
	return result;
    }

    public static String getFilename( String classname ) {
	int pos = classname.lastIndexOf( "." );
	if( pos != -1 )
	    classname = classname.substring( pos + 1 );
	pos = classname.lastIndexOf( "/" );
	if( pos != -1 )
	    classname = classname.substring( pos + 1 );
	return classname + ".java";
    }

    public void invokeTarget( Method main, String[] args )
    throws IllegalAccessException, InvocationTargetException {
	main.invoke( null, new Object[] { args } );	
    }

    public Debugger( String[] args ) {
        int initialLine = 0;
	String initialFile = "";
        String classname = "";
	String filelist = " ";
        String arguments[] = new String[] { };
	Vector argumentVector = new Vector( );
        int port = 0;
        String host = "";
	boolean prompt = false;
	int bufferSize = 128 * 1024;
	boolean useVector = false;
	String deepLogList = "java.util.*";
	
        try {
            port = Integer.parseInt( args[ 0 ] );
            host = args[ 1 ];
        } catch( Exception e ) {
            usage( );
        }
        clientSocket = new ClientSocket( port, host );
        Breakpoints breakpoints = new Breakpoints( clientSocket );
	Bridge.setBreakpoints( breakpoints );
	
        for( ; ; ) {
            if( prompt ) 
		System.out.print( "\nBdbj >" );
            String command = clientSocket.readLine( );
	    if( command.startsWith( "+ " ) )
		command = command.substring( 2 );
	    if( command.equals( "" ) )
		;
	    else if( command.startsWith( "class" ) ) {
		classname = Decoder.getString( command );
	    }
	    else if( command.startsWith( "file" ) )
		filelist += Decoder.getString( command ) + " ";
            else if( command.startsWith( "args" ) )
                arguments = getStringArray( Decoder.getString( command ) );
	    else if( command.startsWith( "deeplog" ) )
		deepLogList = Decoder.getString( command );
	    else if( command.startsWith( "arg" ) ) {
		argumentVector.add( Decoder.getString( command ) );
		useVector = true;
	    }
            else if( command.startsWith( "init" ) ) {
		try {
		    String end = command.substring( 5 ).trim( );
		    int pos = end.indexOf( ' ' );
		    int line;
		    String file;
		    if( pos == -1 ) {
			line = Integer.parseInt( end );
			file = getFilename( classname );
		    } else {
			line = Integer.parseInt( end.substring( 0, end.indexOf( ' ' ) ) );
			file = end.substring( end.indexOf( ' ' ) + 1 ).trim( );
		    }
		    if( line == -1 )
			clientSocket.writeLine( "error: Invalid Initial Line" );
		    else {
			initialLine = line;
			initialFile = file;
		    }
		} catch( Exception e ) {
		    clientSocket.writeLine( "error: Bad Input" );
		}
            } else if( command.startsWith( "buffer" ) ) {
		int val = Decoder.getNumber( command );
		if( val < 8 )
		    clientSocket.writeLine( "error: Bad Buffer Size" );
		else
		    bufferSize = val;
	    } else if( command.equals( "end" ) ||
                     command.equals( "quit" ) ||
                     command.equals( "exit" ) ||
                     command.equals( "terminate" ) ) {
                clientSocket.close( );
                System.exit( 0 );
            } else if( command.equals( "start" ) )
                break;
            else if( command.equals( "help" ) )
                help( );
	    else if( command.equals( "prompt" ) ) {
		prompt = true;
		clientSocket.setPrompt( true );
	    } else if( command.equals( "testing" ) ) {
		// bit of a hack: if we are testing we don't want the prompt 
		// to appear in the output so the Interpreter is told that there is no prompt;
		// but we do want to see both stdout and the replies, so clientSocket is
		// told that there is a prompt.
		prompt = false;
		clientSocket.setPrompt( true );
	    }
	    else if( command.startsWith( "toggle" ) ) {
		try {
		    String end = command.substring( 7 ).trim( );
		    int pos = end.indexOf( ' ' );
		    int line;
		    String file;
		    if( pos == -1 ) {
			line = Integer.parseInt( end );
			file = getFilename( classname );
		    } else {
			line = Integer.parseInt( end.substring( 0, end.indexOf( ' ' ) ) );
			file = end.substring( end.indexOf( ' ' ) + 1 ).trim( );
		    }
		    if( line < 0 )
			clientSocket.writeLine( "error: Bad Line Number" );
		    else
			breakpoints.toggleBreakpoint( line, file );
		} catch( Exception e ) {
		    clientSocket.writeLine( "error: Bad Input" );
		}
	    }
	    else if( command.equals( "listbpts" ) )
		breakpoints.listBreakpoints( );
	    else if( command.equals( "clearallbpts" ) )
		breakpoints.clearAllBreakpoints( );
            else
                clientSocket.writeLine( "error: Unrecognised Command" );
        }

	if( useVector )
	    arguments = vectorToStringArray( argumentVector );

	if( filelist.equals( " " ) )
	    filelist = " " + classname + ".java ";

        interpreter = new Interpreter( clientSocket, prompt, classname, initialLine, 
				       initialFile, filelist, bufferSize, deepLogList, breakpoints );
        interpreter.start( );

	try {
	    ClassLoader loader = ClassLoader.getSystemClassLoader( );
	    Class clazz = loader.loadClass( classname );
	    Method method = clazz.getDeclaredMethod( "main", 
						     new Class[] { Class.forName( "[Ljava.lang.String;" ) } );
	    invokeTarget( method, arguments );
	} catch( InvocationTargetException e ) {
	    e.getTargetException( ).printStackTrace( );
	} catch( Exception e ) {
	    e.printStackTrace( );
	}   

	// please, do not close the client socket here: there might be other threads
	// still running so we don't want to break the pipe.  
    }
}
