/***********************************************************************
 * Bdbj 1.2 (by Jon Cook May 2002)
 *
 * Watches.java 
 * 
 * Determine whether a watch variable is syntactically valid.
 ***********************************************************************/

package jjc.bdbj.client;

public class Watches {
    private static boolean isValidIndex( String string ) {
        // check we conform to the following:
        // <int>::- [0-9]+
        if( string.length( ) == 0 )
            return false;
	if( string.length( ) > 2 &&
	    string.length( ) <= 6 &&
	    string.charAt( 0 ) == '0' &&
	    Character.toUpperCase( string.charAt( 1 ) )== 'X' ) {
	    for( int i = 2; i < string.length( ); i++ ) {
		char next = Character.toUpperCase( string.charAt( i ) );
		if( !Character.isDigit( next ) &&
		    next < 'A' && next > 'F' )
		    return false;
	    }
	    return true;
	}
	if( string.length( ) == 3 &&
	    string.charAt( 0 ) == '\'' &&
	    string.charAt( 2 ) == '\'' )
	    return true;
	for( int i = 0; i < string.length( ); i++ )
	    if( !Character.isDigit( string.charAt( i ) ) )
		return false;
        return true;
    }

    private static boolean isValidSubIdentifier( String string ) {
        // check we conform to the following:
        // <lid>::- ' '* <start>( <part> )* ( '[' ' '* (<id> | <int>) ' '* ']' )* ' '*
	string = string.trim( );

        if( string.length( ) == 0 )
            return false;
        if( !Character.isJavaIdentifierStart( string.charAt( 0 ) ) )
            return false;
        int pos = string.indexOf( '[' );
        String leader;
        if( pos == -1 )
            leader = string.substring( 1 );
        else
            leader = string.substring( 1, pos );

	/* trim only at end otherwise "a b" slips through */
	while( leader.length( ) > 0 && 
	       leader.charAt( leader.length( ) - 1 ) == ' ' )
	    leader = leader.substring( 0, leader.length( ) - 1 );
        for( int i = 0; i < leader.length( ); i ++ )
            if( !Character.isJavaIdentifierPart( leader.charAt( i ) ) )
                return false;
        if ( pos == -1 )
            return true;
        string = string.substring( pos ).trim( );
        for( ; ; ) {
            if( string.length( ) == 0 )
                return true;
            if( string.charAt( 0 ) != '[' )
                return false;
	    int count = 1;
	    if( string.indexOf( ']' ) == -1 )
		return false;
	    pos = 1;
	    while( count != 0 ) {
		if( string.charAt( pos ) == '[' )
		    count++;
		else if( string.charAt( pos ) == ']' )
		    count--;
		pos++;
	    }
	    pos--;
            if( !isValidIndex( string.substring( 1, pos ).trim( ) ) 
		&& !isValidIdentifier( string.substring( 1, pos ).trim( ) ) )
                return false;
            string = string.substring( pos + 1 ).trim( );
        }
    }

    public static boolean isValidIdentifier( String string ) {
        // check we conform to the folowing:
        // <id>::- <lid> | <lid> '.' <id>
        String splitter = string.trim( );
	if( splitter.length( ) > 0 && splitter.charAt( 0 ) == '!' )
	    return true;
        int pos;
        while( ( pos = splitter.indexOf( '.' ) ) != -1 ) {
            if( !isValidSubIdentifier( splitter.substring( 0, pos ) ) )
                return false;
            splitter = splitter.substring( pos + 1 );
        }
        if( !isValidSubIdentifier( splitter ) )
            return false;
        return true;
    }
}
