/***********************************************************************
 * Bdbj 1.2 (by Jon Cook May 2002)
 *
 * ModeToolBar.java 
 * 
 * Allows creation of a tool bar, where items and whole menus are
 * automatically enabled and disabled depending on the current mode
 * of the application.
 ***********************************************************************/

package jjc.bdbj.interfaces.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class ModeToolBar extends JToolBar implements Runnable {
    protected ToolBarData toolBarData; 
    protected JButton[] jButtons;
    protected String[] modes;
    private String currentMode;

    public ModeToolBar( ToolBarData toolBarData, String[] modes ) {
        this.toolBarData = toolBarData;
        this.modes = modes;
	setName( toolBarData.title );
	this.toolBarData = toolBarData;
	int numButtons = toolBarData.items.length;
	jButtons = new JButton[ numButtons ];

	for( int i = 0; i < numButtons; i++ ) {
	    ToolBarButtonData toolBarButtonData = toolBarData.items[ i ];
	    if( toolBarButtonData.internalName.equals( "" ) ) {
		addSeparator( );
		jButtons[ i ] = null;
	    }
	    else {
		add( jButtons[ i ] = new JButton
			      ( new ImageIcon
				( ModeToolBar.class.getResource
				  ( toolBarButtonData.resource ) ) ) );
		jButtons[ i ].addActionListener( toolBarButtonData.handler );
		jButtons[ i ].setActionCommand( toolBarButtonData.internalName );
		jButtons[ i ].setToolTipText( toolBarButtonData.internalName );
		jButtons[ i ].setBackground( new Color( 0.7f, 0.7f, 0.7f ) );
	    }
        }
	setFloatable( false ); 
    }

    private int givenMode;
    private void setGivenMode( int givenMode ) {
	this.givenMode = givenMode;
    }

    public void run( ) {
	int numItems = toolBarData.items.length;
	for( int item = 0; item < numItems; item++ ) 
	    if( jButtons[ item ] != null )
		jButtons[ item ].setEnabled( toolBarData.items[ item ].enabledForMode[ givenMode ] );
    }

    public synchronized void changeMode( String newMode ) {
        if( modes == null ) 
            return;
        int givenMode = -1; // Error value
        int numModes = modes.length;
        for( int mode = 0; mode < numModes; mode++ )
            if( modes[ mode ].equals( newMode ) )
                givenMode = mode;
        if( givenMode == -1 ) {
            System.err.println( "Bad mode given to ToolBar changeMode" );
            return;
        }
	setGivenMode( givenMode );
	if( SwingUtilities.isEventDispatchThread( ) ) 
	    run( );
	else {
	    try {
		SwingUtilities.invokeAndWait( this );
	    } catch( Exception e ) { }
	}		    
        currentMode = newMode;
    }

    public synchronized String getCurrentMode( ) {
        return currentMode;
    }
}
