/***********************************************************************
 * Bdbj 1.2 (by Jon Cook May 2002)
 *
 * ThreadManager.java 
 * 
 * Maintain a visible list of threads, allow them to be selected and
 * call FileManager to change and move to current lines of execution.
 ***********************************************************************/

package jjc.bdbj.interfaces.gui;

import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

public class ThreadManager extends Vector implements ListSelectionListener {

    private FileManager fileManager = null;
    private static JList threadList = null;
    private int currentThread = 0;
    private boolean bytecode = false;

    public ThreadManager( ) {
	threadList = new JList( this );
	threadList.addListSelectionListener( this );
	threadList.addMouseListener( new MouseAdapter( ) {
		public void mouseClicked( MouseEvent e ) {
		    valueChanged( );
		}
	    } );
    }			     
    
    public JList getThreadList( ) {
	return threadList;
    }
                                           
    public void setFileManager( FileManager fileManager ) {
	this.fileManager = fileManager;
    }

    private MessageReceiver messageReceiver = null;
    public void setMessageReceiver( MessageReceiver messageReceiver ) {
	this.messageReceiver = messageReceiver;
    }

    public void valueChanged( ListSelectionEvent event ) {
	valueChanged( );
    }

    public void valueChanged( ) {
	DebugThread o = (DebugThread)threadList.getSelectedValue( );
	if( o != null ) {
	    String str = o.fullString( );
	    int pos = str.indexOf( " " );
	    if( bytecode ) {
		if( o.file != null ) {
		    fileManager.jumpToThreadHighlight( o.currentLine, o.file  );
		    bytecode = false;
		} else if( o.bytecodeFile != null ) 
		    fileManager.jumpToThreadHighlight( o.currentPc, o.currentMethod, o.bytecodeFile );
	    } else {
		if( o.bytecodeFile != null ) {
		    fileManager.jumpToThreadHighlight( o.currentPc, o.currentMethod, o.bytecodeFile );
		    bytecode = true;
		} else if( o.file != null )
		    fileManager.jumpToThreadHighlight( o.currentLine, o.file  );
	    }
	    messageReceiver.deliverMessage( "thread " + str.substring( 0, pos ) );
	    currentThread = Integer.parseInt( str.substring( 0, pos ) );
	}  
    }

    public synchronized void newThread( String id ) {
	DebugThread dt = null;
	Object o = threadList.getSelectedValue( );
	addElement( dt = new DebugThread( fileManager, id ) );
	threadList.setListData( this );
	if( o == null )
	    threadList.setSelectedValue( dt, true );
	else
	    threadList.setSelectedValue( o, true );
    }

    public synchronized void threadDead( String id ) {
	DebugThread dt = null;
	for( Enumeration elts = elements( );
	     elts.hasMoreElements( );
	     ) {
	    dt = (DebugThread)elts.nextElement( );
	    if( dt.fullString( ).equals( id ) ) {
		dt.removeCurrentLine( );
		Object o = threadList.getSelectedValue( );		
 		removeElement( dt );
		threadList.setListData( this );
		threadList.repaint( );
		if( o != dt )
		    threadList.setSelectedValue( o, true );
		else if( !isEmpty( ) )
		    threadList.setSelectedValue( firstElement( ), true );
		valueChanged( );
		return;
	    }
	}
    }

    public synchronized void changeCurrentLine( int line, String file, int thr, boolean gotoLine ) {
	DebugThread dt = null;
	for( Enumeration elts = elements( );
	     elts.hasMoreElements( );
	     ) {
	    dt = (DebugThread)elts.nextElement( );
	    if( dt.getNumber( ) == thr ) 
		dt.setCurrentLine( line, file, currentThread == thr, gotoLine );
	}
    }

    public synchronized void changeCurrentPc( int pc, String method, String file, int thr, boolean gotoLine ) {
	DebugThread dt = null;
	for( Enumeration elts = elements( );
	     elts.hasMoreElements( );
	     ) {
	    dt = (DebugThread)elts.nextElement( );
	    if( dt.getNumber( ) == thr ) 
		dt.setCurrentPc( pc, method, file, currentThread == thr, gotoLine );
	}
    }

    public synchronized void removeCurrentLines( ) {
	for( Enumeration elts = elements( );
	     elts.hasMoreElements( );
	     )
	    ((DebugThread)(elts.nextElement( ))).removeCurrentLine( );
    }

    public synchronized void removeAllThreads( ) {
	removeAllElements( );
	threadList.setListData( this );
    }
}

class DebugThread {
    private int number;
    private String fullname;
    public int currentLine = 0;
    public String file = null;
    private FileManager fileManager = null;

    public int currentPc = -1;
    public String currentMethod = "";
    public String bytecodeFile = null;

    public DebugThread( FileManager fileManager, String fullname ) {
	this.fileManager = fileManager;
	this.fullname = fullname;
	this.number = Integer.parseInt( fullname.substring( 0, fullname.indexOf( ' ' ) ) );
    }

    public int getNumber( ) {
	return number;
    }

    public void removeCurrentLine( ) {
	if( currentLine != 0 ) 
	    fileManager.removeThreadHighlight( this.currentLine, this.file );	    
	if( currentPc != -1 )
	    fileManager.removeThreadHighlight( this.currentPc, this.currentMethod, this.bytecodeFile );

	currentLine = 0;
	currentPc = -1;
	currentMethod = "";
	file = null;
	bytecodeFile = null;
    }

    public void setCurrentLine( int line, String file, boolean current, boolean gotoLine ) {
	if( currentLine != 0 ) 
	    fileManager.removeThreadHighlight( this.currentLine, this.file );	    
	this.currentLine = line;
	this.file = file;
	fileManager.addThreadHighlight( currentLine, file, current, gotoLine );
    }

    public void setCurrentPc( int pc, String method, String file, boolean current, boolean gotoLine ) {
	if( currentPc != -1 ) 
	    fileManager.removeThreadHighlight( this.currentPc, this.currentMethod, this.bytecodeFile );
	this.currentPc = pc;
	this.currentMethod = method;
	this.bytecodeFile = file;
	fileManager.addThreadHighlight( pc, method, file, current, gotoLine );
    }

    public int getCurrentLine( ) {
	return currentLine;
    }

    public String fullString( ) {
	return fullname;
    }

    public String toString( ) {
	return fullname.substring( fullname.indexOf( ' ' ) + 1 );
    }
}
