/***********************************************************************
 * Bdbj 1.2 (by Jon Cook May 2002)
 *
 * TTY.java 
 * 
 * This is the teletype user interface for Bdbj.
 ***********************************************************************/

package jjc.bdbj.interfaces.tty;

import java.io.*;
import java.net.*;

public class TTY {
    public static void main( String[] args ) {
	new TeletypeInterface( args );
    }
}

class TeletypeInterface {

    private String className = "";
    private String arguments = "";
    private int initial = 0;

    private String argsToString( String[] args, int nextArg ) {
	String string = "";
	for( int i = nextArg; i < args.length; i++ ) 
	    if( i == args.length - 1 )
		string += args[ i ];
	    else
		string += args[ i ] + " ";
	return string;
    }

    private void usage( ) {
	System.out.println( "Bdbj 1.2 (2002) by Jon Cook" );
        System.out.println( "Usage: Graphical:    bdbj    <classname> [<files>]" );
        System.out.println( "       Command Line: bdbjtty <classname> [<files>]" );
        System.exit( 1 );
    }

    public TeletypeInterface( String[] args ) {
	if( args.length < 1 )
	    usage( );

	Notifier notifier = new Notifier( );
	int nextArg = 0;
	boolean gcOn = true;
	if( args[ nextArg ].equals( "-nogc" ) ) {
	    gcOn = false;
	    nextArg++;
	}
	className = args[ nextArg++ ];
	
	arguments = "";

	String[] debugFiles = new String[ args.length - nextArg ];
	System.arraycopy( args, nextArg, debugFiles, 0, debugFiles.length );

	TTYServerSocket serverSocket =
	  new TTYServerSocket( className, arguments, initial, gcOn, debugFiles, notifier );
	serverSocket.start( );
	
	BufferedReader in = new BufferedReader( new InputStreamReader( System.in ) );
	notifier.waitfor( );

	String next;
	for( ; ; ) {
	    try {
		next = in.readLine( );
	    } catch( IOException _ ) {
		next = "";
	    }
	    if( next == null ) {
		serverSocket.send( "quit" );
		System.out.println( );
		System.exit( 0 );
	    } else {
		if( next.trim( ).equals( "" ) )
		    System.out.print( "\nBdbj >" );
		else {	
		    if( next.equals( "waitquit" ) ) {
			notifier.waitfor( );
			break;
		    }			 
		    else if( next.trim( ).equals( "wait" ) ) {
			try {
			    Thread.sleep( 1000L );
			} catch( InterruptedException _ ) { }	   
		    }
		    else if( next.startsWith( "input " ) ) {
			serverSocket.sendStdin( next.substring( 6 ) );
			System.out.print( "\nBdbj >" );
		    } else if( next.equals( "input" ) ) {
			serverSocket.sendStdin( "" );
			System.out.print( "\nBdbj >" );
		    } else {
			serverSocket.send( next.trim( ) );
			if( next.trim( ).equals( "exit" ) ||
			    next.trim( ).equals( "terminate" ) ||
			    next.trim( ).equals( "end" ) ||
			    next.trim( ).equals( "quit" ) ) {
			    System.out.println( );
			    System.exit( 0 );
			}
		    }
		}
	    }
	}
    }		 
}

class TTYServerSocket extends Thread {
    public int DEFAULT_SOCKET = 24210;
    private static String sep = System.getProperty( "file.separator" );

    private Process process;
    private ServerSocket serverSocket = null;
    private Socket socket = null;

    private String kaffeLocation = System.getProperty( "bdbj.kaffedir" );
    private String classPath = System.getProperty( "bdbj.classpath" );
    private String kaffeOptions = System.getProperty( "bdbj.options" ).replace( '_', '.' );
    private String classname;
    private String args;
    private int initial;
    private boolean gcOn;
    private String[] debugFiles;
    
    private static ConnectStreams stdout = null;
    private static ConnectStreams stderr = null;
    private PrintWriter socketOut = null;
    private PrintWriter processOut = null;

    private Notifier notifier;

    public synchronized void send( String command ) {
	// synchronized for two reasons:
	// (i) socketOut is initialised by another thread
	// (ii) we don't want to allow two messages to get mixed together
	if( socketOut != null ) {
	    socketOut.println( command );
	    socketOut.flush( );
	}
    }

    public synchronized void sendStdin( String input ) {
	if( processOut != null ) {
	    processOut.println( input );
	    processOut.flush( );
	}
    }

    private synchronized void setOut( OutputStream socketOut,
				      OutputStream processOut ) {
	this.socketOut = new PrintWriter( new OutputStreamWriter( socketOut ) );
	this.processOut = new PrintWriter( new OutputStreamWriter( processOut ) );
    }

    private String getClassname( String filename ) {
	String separator = System.getProperty( "file.separator" );
	String end;
	int lastSep = filename.lastIndexOf( separator );
	if( lastSep == -1 )
	    end = filename;
	else
	    end = filename.substring( lastSep + separator.length( ) );
	if( end.endsWith( ".java" ) )
	    return end.substring( 0, end.length( ) - ".java".length( ) );
	else
	    return "";
    }

    public TTYServerSocket( String classname, String args,
			    int initial, boolean gcOn, String[] debugFiles, Notifier notifier ) {
	this.classname = classname;
	this.args = args;
	this.initial = initial;
	this.gcOn = gcOn;
	this.debugFiles = debugFiles;
	if( kaffeLocation == null )
	    kaffeLocation = ".";
	if( classPath == null )
	    classPath = "";
	if( kaffeOptions == null )
	    kaffeOptions = "";    
	this.notifier = notifier;
    }

    public void run( ) {
	try {
	    // create the server socket
	    try {
		serverSocket = new ServerSocket( DEFAULT_SOCKET );
	    } catch( Exception e ) {
		serverSocket = new ServerSocket( 0 );
	    }

	    process = Runtime.getRuntime( ).exec( kaffeLocation + sep + "bin" + sep + "kaffe -jar " 
						  + ( gcOn ? "" : "-nogcJJC " )
						  + ( classPath.trim( ).equals( "" ) ? "" : "-classpath " 
						      + classPath + " " )
						  + kaffeOptions + " -debugJJC "
						  + kaffeLocation + sep + "bdbj" + sep + "Bdbj.jar "
						  + serverSocket.getLocalPort( ) + " " 
						  + serverSocket.getInetAddress( ).getHostName( ) );
	    
	    // wait for connection, abort if takes longer than 1 minute
	    serverSocket.setSoTimeout( 60 * 1000 ); 
	    socket = serverSocket.accept( );

	    stdout = new ConnectStreams( process.getInputStream( ), System.out );
	    stderr = new ConnectStreams( process.getErrorStream( ), System.err );
	    setOut( socket.getOutputStream( ), process.getOutputStream( ) );

	    stdout.start( );
	    stderr.start( );

	    send( "class " + classname );
	    send( "args " + args );
	    for( int i = 0; i < debugFiles.length; i++ )
		send( "file " + debugFiles[ i ] );
	    send( "prompt" );

	    notifier.pulse( );

	    process.waitFor( );
	    notifier.pulse( );
	} catch( Exception e ) {
	    System.out.println( "Exception with socket = " + e );
	} finally {
	    try {
		serverSocket.close( );
		socket.close( );
	    } catch( Exception _ ) { }
	}

    }
}

class ConnectStreams extends Thread {
    // this thread is used to copy the client's stderr/stdout to this program's stdout.
    private InputStream input = null;
    private OutputStream output = null;

    public ConnectStreams( InputStream input, OutputStream output ) {
	this.input = input;
	this.output = output;
    }

    public void run( ) {
	int available;
	try {
	    for( ; ; ) {
		int data = input.read( );
		if( data == -1 )
		    break;
		output.write( data );
		output.flush( );
	    }
	} catch( Exception _ ) { }
    }
}

class Notifier {

    private boolean pulsed = false;

    public synchronized void waitfor( ) {
	try {
	    while( !pulsed )
		wait( );
	} catch( InterruptedException _ ) {
	}
	pulsed = false;
    }

    public synchronized void pulse( ) {
	pulsed = true;
	notify( );
    }
}
